@@ -23,7 +23,7 @@ Created on Nov 11, 2010
 @author: tmetsch
 '''
 
-from pyocci import registry
+from pyocci import registry, service
 from pyocci.core import Category, Kind, Resource, Link, Action, Mixin
 from pyocci.my_exceptions import ParsingException
 import re
@@ -91,7 +91,13 @@ class Rendering(object):
         @param body: The HTTP body.
         @type body: str
         '''
-        raise NotImplementedError
+        raise NotImplementedError()
+
+    def login_information(self):
+        '''
+        Return proper information indicating how the user can login.
+        '''
+        raise NotImplementedError()
 
     def to_action(self, headers, data):
         '''
@@ -364,6 +370,13 @@ class TextPlainRendering(Rendering):
             raise ParsingException('Body does not contain any X-OCCI-Locations'
                                    + ' pointing to resource instances.')
 
+    def login_information(self):
+        html = 'Please do a POST operation with a name and pass attribute.'
+
+        headers = {}
+        headers['Content-Type'] = self.content_type
+        return headers, html
+
     def to_action(self, headers, body):
         data = self._get_data(body)
 
@@ -542,6 +555,13 @@ class TextHeaderRendering(Rendering):
         else:
             raise ParsingException('Header does not contain a location.')
 
+    def login_information(self):
+        html = 'Please do a POST operation with a name and pass attribute.'
+
+        headers = {}
+        headers['Content-Type'] = self.content_type
+        return headers, html
+
     def to_action(self, headers, body):
         data = self._get_data(headers)
 
@@ -635,45 +655,55 @@ class TextHTMLRendering(Rendering):
         css_string = 'body {font: .8em/normal sans-serif;}'
         css_string += 'div {padding: 1em; margin: 1em;'
         css_string += 'border: 1px solid #73c167;}'
+        css_string += 'table {margin: 1em; border: 1px solid #444;}'
 
     def from_categories(self, categories):
         html = self._create_html_head()
 
         html += '<h1>Registered Categories</h1>'
 
+        sorted_cats = {}
+
         if len(categories) > 0:
             for cat in categories:
-                html += '<table>'
-                html += '<tr><th>Term'
-                html += '</th><td>'
-                html += cat.term + '</td></tr>'
-                html += '<tr><th>Scheme'
-                html += '</th><td>'
-                html += cat.scheme + '</td></tr>'
+                tmp = '<table>'
+                tmp += '<tr><th>Term'
+                tmp += '</th><td>'
+                tmp += cat.term + '</td></tr>'
+                tmp += '<tr><th>Scheme'
+                tmp += '</th><td>'
+                tmp += cat.scheme + '</td></tr>'
 
                 if hasattr(cat, 'title') and cat.title is not '':
-                    html += '<tr><th>Title'
-                    html += '</th><td>'
-                    html += cat.title + '</td></tr>'
+                    tmp += '<tr><th>Title'
+                    tmp += '</th><td>'
+                    tmp += cat.title + '</td></tr>'
                 if hasattr(cat, 'actions') and len(cat.actions) > 0:
-                    html += '<tr><th>Actions'
-                    html += '</th><td>'
-                    html += repr(cat.actions) + '</td></tr>'
+                    tmp += '<tr><th>Actions'
+                    tmp += '</th><td>'
+                    tmp += repr(cat.actions) + '</td></tr>'
                 if hasattr(cat, 'attributes')and len(cat.attributes) > 0:
-                    html += '<tr><th>Attributes'
-                    html += '</th><td>'
-                    html += ','.join(cat.attributes) + '</td></tr>'
+                    tmp += '<tr><th>Attributes'
+                    tmp += '</th><td>'
+                    tmp += ','.join(cat.attributes) + '</td></tr>'
                 if hasattr(cat, 'location') and cat.location is not '':
-                    html += '<tr><th>Location'
-                    html += '</th><td><a href="'
-                    html += cat.location + '">'
-                    html += cat.location + '</a></td></tr>'
+                    tmp += '<tr><th>Location'
+                    tmp += '</th><td><a href="'
+                    tmp += cat.location + '">'
+                    tmp += cat.location + '</a></td></tr>'
                 if hasattr(cat, 'related')and len(cat.related) > 0:
-                    html += '<tr><th>related'
-                    html += '</th><td>'
-                    html += repr(cat.related) + '</td></tr>'
+                    tmp += '<tr><th>related'
+                    tmp += '</th><td>'
+                    tmp += repr(cat.related) + '</td></tr>'
 
-                html += '</table>'
+                tmp += '</table>'
+                if sorted_cats.has_key(cat.scheme):
+                    sorted_cats[cat.scheme] = sorted_cats[cat.scheme] + tmp
+                else:
+                    sorted_cats[cat.scheme] = tmp
+            for scheme in sorted_cats:
+                html += '<h2>' + scheme + '</h2>'
+                html += sorted_cats[scheme]
         else:
             html += '<p>No categories registered</p>'
 
@@ -765,6 +795,7 @@ class TextHTMLRendering(Rendering):
                 html += entity.attributes[item] + '</td></tr>'
             html += '</table>'
 
+        html += '<h2>&nbsp;</h2>'
         html += '<input type="button" value="Back" onclick="goBack()" />'
 
         html += self._create_html_footer()
@@ -777,6 +808,20 @@ class TextHTMLRendering(Rendering):
         # This is something HTML rendering cannot do...
         raise NotImplementedError('HTML rendering does not support this.')
 
+    def login_information(self):
+        html = self._create_html_head()
+        html += '<h1>Please Sign in</h1>'
+        html += '<form action="/login" method="post">'
+        html += 'Username: <input type="text" name="name"><br />'
+        html += 'Password: <input type="password" name="pass"><br />'
+        html += '<input type="submit" value="Sign in">'
+        html += '</form>'
+        html += self._create_html_footer()
+
+        headers = {}
+        headers['Content-Type'] = self.content_type
+        return headers, html
+
     def to_action(self, headers, body):
         if body is not None:
             tmp = urllib.unquote(body).split('&')
@@ -912,6 +957,8 @@ class TextHTMLRendering(Rendering):
         html += '<div id="header">'
         html += '    <a href="/">Home</a>'
         html += '    <a href="/-/">Query Interface</a>'
+        if service.AUTHENTICATION is True:
+            html += '    <a href="/logout">Logout</a>'
         html += '</div>'
         html += '<div id="content">'
         return html
